class UnresolvedAlgorithmTypeError(Exception):
    pass


class UnusedStrictUrlParameterError(Exception):
    pass


class IncorrectUrlMethodError(Exception):
    pass


class RobokassaInterfaceError(Exception):
    pass



from robokassa.asyncio.connection import Requests
from robokassa.utils import HttpResponseValidator


class AsyncMerchant:
    def __init__(self, http: Requests, merchant_login: str) -> None:
        self._http = http.connection

        self._merchant_login = merchant_login

    async def get_currencies(self, language: str) -> dict:
        async with self._http as conn:
            response = await conn.post(
                url="WebService/Service.asmx/GetCurrencies",
                data={
                    "MerchantLogin": self._merchant_login,
                    "Language": language,
                },
            )
            validated_response = HttpResponseValidator(
                response, False
            ).validate_http_response()
            return validated_response



from typing import Optional

from httpx import AsyncClient

from robokassa.connection import BaseHttpConnection, BaseRequests


class AsyncHttpConnection(BaseHttpConnection):
    def __init__(self, base_url: str = "") -> None:
        self.base_url: str = base_url

        self._async_client: Optional[AsyncClient] = None

    async def __aenter__(self) -> AsyncClient:
        self._async_client = AsyncClient(base_url=self.base_url)
        return self._async_client

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        await self._async_client.aclose()


class Requests(BaseRequests):
    _base_url = "https://auth.robokassa.ru/Merchant"
    connection = AsyncHttpConnection(base_url=_base_url)



from typing import Optional

from httpx import Client


class BaseHttpConnection:
    pass


class HttpConnection(BaseHttpConnection):
    def __init__(self, base_url: str = "") -> None:
        self.base_url: str = base_url

        self._sync_client: Optional[Client] = None

    def __enter__(self) -> Client:
        self._sync_client = Client(base_url=self.base_url)
        return self._sync_client

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        self._sync_client.close()


class BaseRequests:
    pass


class Requests(BaseRequests):
    _base_url = "https://auth.robokassa.ru/Merchant"
    connection = HttpConnection(base_url=_base_url)



from typing import Union

from robokassa.connection import Requests
from robokassa.utils import HttpResponseValidator


class BaseMerchant:
    pass


class Merchant(BaseMerchant):
    def __init__(self, http: Requests, merchant_login: str) -> None:
        self._http = http.connection

        self._merchant_login = merchant_login

    def get_currencies(self, language: str) -> dict:
        with self._http as conn:
            response = conn.post(
                url="WebService/Service.asmx/GetCurrencies",
                data={
                    "MerchantLogin": self._merchant_login,
                    "Language": language,
                },
            )
            validated_response = HttpResponseValidator(
                response, False
            ).validate_http_response()
            return validated_response

    def get_operation_state(
        self, signature_value: str, invoice_id: Union[int, str]
    ) -> dict:
        """
        Get state of operation.
        :param signature_value: MerchantLogin:InvoiceID:Password#2
        :param invoice_id: Store account number
        :return: dict
        """
        with self._http as conn:
            response = conn.post(
                url="WebService/Service.asmx/OpStateExt",
                data={
                    "MerchantLogin": self._merchant_login,
                    "InvoiceID": invoice_id,
                    "Signature": signature_value,
                },
            )
            validated_response = HttpResponseValidator(
                response, False
            ).validate_http_response()

            return validated_response



from typing import Optional, Union

from robokassa.connection import Requests
from robokassa.exceptions import (
    UnusedStrictUrlParameterError,
    IncorrectUrlMethodError,
)
from robokassa.hash import HashAlgorithm, Hash
from robokassa.merchant import Merchant
from robokassa.payment import Payment


class RobokassaAbstract:
    pass


class BaseRobokassa(RobokassaAbstract):
    def __init__(
        self,
        merchant_login: str,
        password1: str,
        password2: str,
        algorithm: HashAlgorithm = HashAlgorithm.md5,
        is_test: bool = False,
        use_standard_naming_of_additional_link_params: bool = True,
    ) -> None:
        self._merchant_login = merchant_login
        self._password1 = password1
        self._password2 = password2
        self._algorithm = algorithm
        self._is_test = is_test

        self._use_standard_naming_of_additional_link_params = (
            use_standard_naming_of_additional_link_params
        )

        self._hash = self._init_hash(self._algorithm)

    def _init_hash(self, algorithm: HashAlgorithm) -> Hash:
        return Hash(algorithm)

    @property
    def merchant_login(self) -> str:
        return self._merchant_login

    @property
    def algorithm(self) -> HashAlgorithm:
        return self._algorithm

    @property
    def is_test(self) -> bool:
        return self._is_test

    def __repr__(self):
        return f"{self.__class__.__name__}(is_test={self.is_test})"


class Robokassa(BaseRobokassa):
    def __init__(
        self,
        merchant_login: str,
        password1: str,
        password2: str,
        algorithm: HashAlgorithm = HashAlgorithm.md5,
        is_test: bool = False,
        use_standard_naming_of_additional_link_params: bool = True,
    ) -> None:
        super().__init__(
            merchant_login=merchant_login,
            password1=password1,
            password2=password2,
            algorithm=algorithm,
            is_test=is_test,
            use_standard_naming_of_additional_link_params=use_standard_naming_of_additional_link_params,
        )

        self.__http = self._init_http_connection()

        self._payment = self._init_payment(
            http=self.__http,
            is_test=is_test,
            merchant_login=self.merchant_login,
            password1=self._password1,
            password2=self._password2,
            hash_=self._hash,
        )
        self._merchant = self._init_merchant(
            self.__http,
            self._merchant_login,
        )

        self._link = self._payment.link
        self._checker = self._payment.check

    def _init_merchant(
        self,
        http: Requests,
        merchant_login: str,
    ) -> Merchant:
        return Merchant(
            http=http,
            merchant_login=merchant_login,
        )

    def _init_payment(
        self,
        http: Requests,
        is_test: bool,
        merchant_login: str,
        password1: str,
        password2: str,
        hash_: Hash,
    ) -> Payment:
        return Payment(
            http=http,
            is_test=is_test,
            merchant_login=merchant_login,
            password1=password1,
            password2=password2,
            hash_=hash_,
        )

    def _init_http_connection(self) -> Requests:
        return Requests()

    def create_link_to_payment_page_by_script(
        self,
        out_sum: float,
        default_prefix: str = "shp",
        result_url: Optional[str] = None,
        success_url: Optional[str] = None,
        success_url_method: Optional[str] = None,
        fail_url: Optional[str] = None,
        fail_url_method: Optional[str] = None,
        inv_id: Optional[int] = 0,
        description: Optional[str] = None,
        **kwargs,
    ) -> str:
        """
        Create a link to payment page by common params with signature.
        Link of this method will look like:


        `https://auth.robokassa.ru/Merchant/Index.aspx?MerchantLogin=demo&OutSum=1&SignatureValue=2c113e992e2c985e43e`

        All link params user can see, but cannot edit them.
        If you want to hide these params you need to use by invoice ID method.


        :param out_sum:
        :param default_prefix: `shp` or `Shp` or `SHP` prefix for additional params
        :param result_url: ResultUrl2
        :param success_url: SuccessUrl2
        :param success_url_method: SuccessUrlMethod2
        :param fail_url: FailUrl2
        :param fail_url_method: FailUrlMethod2
        :param inv_id:
        :param description: Shop description
        :param kwargs: Any additional params without `shp_` prefix
        :return: Link to payment page
        """
        available_http_methods = ("GET", "POST", None)

        if (success_url is not None) != (success_url_method is not None):
            raise UnusedStrictUrlParameterError(
                "If you use URL, you also need to choose a HTTP method"
            )
        if (
            success_url_method not in available_http_methods
            or fail_url_method not in available_http_methods
        ):
            raise IncorrectUrlMethodError("You can use only GET or POST methods")

        payment_link = self._link.generate_by_script(
            default_prefix=default_prefix,
            out_sum=out_sum,
            result_url=result_url,
            success_url=success_url,
            success_url_method=success_url_method,
            fail_url=fail_url,
            fail_url_method=fail_url_method,
            inv_id=inv_id,
            description=description,
            **kwargs,
        )
        return payment_link

    def create_link_to_payment_page_by_invoice_id(
        self,
        inv_id: Optional[Union[str, int]],
        out_sum: Union[str, int, float],
        description: str,
    ) -> str:
        """
        Create a link to payment page by invoice ID.
        It is mean what params will hide.

        Link of this method will look like:

        `https://auth.robokassa.ru/Merchant/Index/41734593-dc97-dc5f-d329-a73158e4cb29`


        :param inv_id:
        :param out_sum:
        :param description: Shop description
        :return: Url to payment page
        """
        return self._link.create_link_to_payment_page_by_invoice_id(
            inv_id=inv_id,
            out_sum=out_sum,
            description=description,
        )

    def success_or_fail_signature_is_valid(
        self,
        signature: str,
        out_sum: Union[str, int, float],
        inv_id: Optional[Union[str, int]],
        **kwargs,
    ) -> bool:
        """
        Check success or fail signature is valid.

        :param signature: Output signature
        :param out_sum:
        :param inv_id:
        :param kwargs: Additional params with `shp_` prefix
        :return: True if signature is valid, else False
        """
        return self._checker.success_or_fail_url_signature_is_valid(
            signature=signature, out_sum=out_sum, inv_id=inv_id, **kwargs
        )

    def result_signature_is_valid(
        self,
        signature: str,
        out_sum: Union[str, int, float],
        inv_id: Optional[Union[str, int]],
        **kwargs,
    ) -> bool:
        """
        Check result signature is valid.

        :param signature: Output signature
        :param out_sum:
        :param inv_id:
        :param kwargs: Additional params with `shp_` prefix
        :return: True if signature is valid, else False
        """
        return self._checker.result_url_signature_is_valid(
            signature=signature, out_sum=out_sum, inv_id=inv_id, **kwargs
        )

    def get_currencies(self, language: str = "en") -> dict:
        """
        Get available currencies of merchant.

        :param language: `ru` or `en`
        :return: dictionary of currencies
        """
        return self._merchant.get_currencies(language=language)

    # def get_operation_state(self, invoice_id: Union[str, int]) -> dict:
    #     signature = Signature(
    #         merchant_login=self._merchant_login,
    #         inv_id=invoice_id,
    #         password=self._password2,
    #         hash_=self._hash,
    #     )
    #     return self._merchant.get_operation_state(
    #         invoice_id=invoice_id, signature_value=signature.value
    #     )



from typing import Union, Any, Optional

from robokassa import HashAlgorithm
from robokassa.asyncio.connection import Requests
from robokassa.asyncio.merchant import AsyncMerchant
from robokassa.asyncio.payment import AsyncPayment
from robokassa.client import BaseRobokassa
from robokassa.exceptions import IncorrectUrlMethodError, UnusedStrictUrlParameterError
from robokassa.hash import Hash


class Robokassa(BaseRobokassa):
    def __init__(
        self,
        merchant_login: str,
        password1: str,
        password2: str,
        algorithm: HashAlgorithm = HashAlgorithm.md5,
        is_test: bool = False,
        use_standard_naming_of_additional_link_params: bool = True,
    ) -> None:
        super().__init__(
            merchant_login,
            password1,
            password2,
            algorithm,
            is_test,
            use_standard_naming_of_additional_link_params,
        )
        self._merchant_login = merchant_login
        self.__password1 = password1
        self.__password2 = password2
        self._is_test = is_test
        self._use_standard_naming = use_standard_naming_of_additional_link_params

        self.__http = self._init_http_connection()

        self._async_payment = self._init_async_payment(
            self.__http,
            is_test=self._is_test,
            merchant_login=self._merchant_login,
            password1=self.__password1,
            password2=self.__password2,
            hash_=self._hash,
        )
        self._async_merchant = self._init_async_merchant(
            self.__http, self._merchant_login
        )

        self._link = self._async_payment.link
        self._checker = self._async_payment.check

    def _init_http_connection(self):
        return Requests()

    def _init_async_payment(
        self,
        http: Requests,
        is_test: bool,
        merchant_login: str,
        password1: str,
        password2: str,
        hash_: Hash,
    ) -> AsyncPayment:
        return AsyncPayment(
            http=http,
            is_test=is_test,
            merchant_login=merchant_login,
            password1=password1,
            password2=password2,
            hash_=hash_,
        )

    def _init_async_merchant(
        self,
        http: Requests,
        merchant_login: str,
    ) -> AsyncMerchant:
        return AsyncMerchant(http=http, merchant_login=merchant_login)

    def create_link_to_payment_page_by_script(
        self,
        out_sum: float,
        default_prefix: str = "shp",
        result_url: Optional[str] = None,
        success_url: Optional[str] = None,
        success_url_method: Optional[str] = None,
        fail_url: Optional[str] = None,
        fail_url_method: Optional[str] = None,
        inv_id: Optional[int] = 0,
        description: Optional[str] = None,
        **kwargs: Any,
    ) -> str:
        """
        Create a link to payment page by common params with signature.
        Link of this method will look like:


        `https://auth.robokassa.ru/Merchant/Index.aspx?MerchantLogin=demo&OutSum=1&SignatureValue=2c113e992e2c985e43e`

        All link params user can see, but cannot edit them.
        If you want to hide these params you need to use by invoice ID method.


        :param out_sum:
        :param default_prefix: `shp` or `Shp` or `SHP` prefix for additional params
        :param result_url: ResultUrl2
        :param success_url: SuccessUrl2
        :param success_url_method: SuccessUrlMethod2
        :param fail_url: FailUrl2
        :param fail_url_method: FailUrlMethod2
        :param inv_id:
        :param description: Shop description
        :param kwargs: any additional params without `shp_` prefix
        :return: link to payment page
        """
        available_http_methods = ("GET", "POST", None)

        if (success_url is not None) != (success_url_method is not None):
            raise UnusedStrictUrlParameterError(
                "If you use URL, you also need to choose a HTTP method"
            )
        if (
            success_url_method not in available_http_methods
            or fail_url_method not in available_http_methods
        ):
            raise IncorrectUrlMethodError("You can use only GET or POST methods")

        return self._link.generate_by_script(
            out_sum=out_sum,
            default_prefix=default_prefix,
            result_url=result_url,
            success_url=success_url,
            success_url_method=success_url_method,
            fail_url=fail_url,
            fail_url_method=fail_url_method,
            inv_id=inv_id,
            description=description,
            **kwargs,
        )

    async def create_link_to_payment_page_by_invoice_id(
        self,
        inv_id: Optional[Union[str, int]],
        out_sum: Union[str, int, float],
        description: str,
    ) -> str:
        """
        Create a link to payment page by invoice ID.
        It is mean what params will hide.

        Link of this method will look like:

        `https://auth.robokassa.ru/Merchant/Index/41734593-dc97-dc5f-d329-a73158e4cb29`


        :param inv_id:
        :param out_sum:
        :param description: Shop description
        :return: Url to payment page
        """
        return await self._link.create_by_invoice_id(
            inv_id=inv_id,
            out_sum=out_sum,
            description=description,
        )

    async def get_currencies(self, language: str = "en") -> dict:
        """
        Get available currencies of merchant.

        :param language: `ru` or `en`
        :return: dictionary of currencies
        """
        return await self._async_merchant.get_currencies(language)

    def success_or_fail_signature_is_valid(
        self,
        signature: str,
        out_sum: Union[str, int, float],
        inv_id: Optional[Union[str, int]],
        **kwargs,
    ) -> bool:
        """
        Check success or fail signature is valid.

        :param signature: Output signature
        :param out_sum:
        :param inv_id:
        :param kwargs: Additional params with `shp_` prefix
        :return: True if signature is valid, else False
        """
        return self._checker.success_or_fail_url_signature_is_valid(
            signature=signature, out_sum=out_sum, inv_id=inv_id, **kwargs
        )

    def result_signature_is_valid(
        self,
        signature: str,
        out_sum: Union[str, int, float],
        inv_id: Optional[Union[str, int]],
        **kwargs,
    ) -> bool:
        """
        Check result signature is valid.

        :param signature: Output signature
        :param out_sum:
        :param inv_id:
        :param kwargs: Additional params with `shp_` prefix
        :return: True if signature is valid, else False
        """
        return self._checker.result_url_signature_is_valid(
            result_signature=signature, out_sum=out_sum, inv_id=inv_id, **kwargs
        )



from typing import Union, Optional

from httpx import Response

from robokassa.asyncio.connection import Requests
from robokassa.hash import Hash
from robokassa.payment import PaymentUrlGenerator
from robokassa.signature import SignaturesChecker
from robokassa.types import RobokassaParams, Signature
from robokassa.utils import HttpResponseValidator


class AsyncPaymentRequests:
    def __init__(self, http: Requests) -> None:
        self._http = http.connection
        self.payment_url = "https://auth.robokassa.ru/Merchant/Index"

    async def _make_post_request(self, data: dict) -> Response:
        async with self._http as conn:
            response = await conn.post("Indexjson.aspx?", data=data)
            return response

    def _serialize_payment_url(self, invoice_id: str) -> str:
        return f"{self.payment_url}/{invoice_id}"

    async def create_url_to_payment_page(
        self, robokassa_params: RobokassaParams
    ) -> str:
        response = await self._make_post_request(robokassa_params.as_dict())
        validated_response = HttpResponseValidator(response).validate_http_response()
        invoice_id = validated_response["invoiceID"]

        return self._serialize_payment_url(invoice_id)


class AsyncPaymentInterface:
    def __init__(self, http: Requests) -> None:
        self._requests = AsyncPaymentRequests(http)

    async def create_url_to_payment_page(
        self, robokassa_params: RobokassaParams
    ) -> str:
        return await self._requests.create_url_to_payment_page(robokassa_params)


class AsyncPaymentLink:
    def __init__(
        self,
        http: Requests,
        is_test: bool,
        hash_: Hash,
        merchant_login: str,
        password1: str,
    ) -> None:
        self._is_test = is_test
        self._hash_ = hash_
        self._merchant_login = merchant_login
        self._password1 = password1

        self._payment_interface = AsyncPaymentInterface(http)

        self._payment_generator = PaymentUrlGenerator(
            is_test=self._is_test,
            hash_=self._hash_,
            merchant_login=self._merchant_login,
            password=password1,
        )

        self.robokassa_params: RobokassaParams = RobokassaParams(
            is_test=self._is_test,
            merchant_login=self._merchant_login,
        )

    def _create_signature(
        self, inv_id: Union[str, int], out_sum: Union[str, int, float]
    ) -> Signature:
        return Signature(
            merchant_login=self._merchant_login,
            password=self._password1,
            inv_id=inv_id,
            out_sum=out_sum,
            hash_=self._hash_,
        )

    def generate_by_script(
        self,
        out_sum: float,
        default_prefix: str = "shp",
        result_url: Optional[str] = None,
        success_url: Optional[str] = None,
        success_url_method: Optional[str] = None,
        fail_url: Optional[str] = None,
        fail_url_method: Optional[str] = None,
        inv_id: Optional[int] = 0,
        description: Optional[str] = None,
        **kwargs,
    ) -> str:
        return self._payment_generator.generate_by_script(
            out_sum=out_sum,
            default_prefix=default_prefix,
            result_url=result_url,
            success_url=success_url,
            success_url_method=success_url_method,
            fail_url=fail_url,
            fail_url_method=fail_url_method,
            inv_id=inv_id,
            description=description,
            **kwargs,
        )

    async def create_by_invoice_id(
        self,
        inv_id: Optional[Union[str, int]],
        out_sum: Union[str, int, float],
        description: str,
    ) -> str:
        return await self._payment_interface.create_url_to_payment_page(
            RobokassaParams(
                is_test=self._is_test,
                merchant_login=self._merchant_login,
                inv_id=inv_id,
                out_sum=out_sum,
                description=description,
                signature_value=self._create_signature(inv_id, out_sum).value,
            )
        )


class AsyncPayment:
    def __init__(
        self,
        http: Requests,
        is_test: bool,
        hash_: Hash,
        merchant_login: str,
        password1: str,
        password2: str,
    ) -> None:
        self._is_test = is_test
        self._hash = hash_
        self._merchant_login = merchant_login
        self._password1 = password1
        self._password2 = password2

        self._http = http

    @property
    def link(self) -> AsyncPaymentLink:
        return AsyncPaymentLink(
            self._http, self._is_test, self._hash, self._merchant_login, self._password1
        )

    @property
    def check(self) -> SignaturesChecker:
        return SignaturesChecker(
            hash_=self._hash,
            password1=self._password1,
            password2=self._password2,
        )



from typing import Dict, Any, Optional, Union
from urllib.parse import urlencode

from httpx import Response

from robokassa.connection import Requests
from robokassa.hash import Hash
from robokassa.signature import SignaturesChecker
from robokassa.types import Signature, RobokassaParams
from robokassa.utils import HttpResponseValidator


class PaymentRequests:
    def __init__(self, http: Requests) -> None:
        self.connection = http
        self.payment_url = "https://auth.robokassa.ru/Merchant/Index"

    def _serialize_payment_url(self, invoice_id: str) -> str:
        return f"{self.payment_url}/{invoice_id}"

    def _make_post_request(self, data: dict) -> Response:
        with self.connection as conn:
            response = conn.post("Indexjson.aspx/?", data=data)
            return response

    def create_url_to_payment_page(self, robokassa_params: RobokassaParams) -> str:
        response = self._make_post_request(robokassa_params.as_dict())
        validated_response = HttpResponseValidator(response).validate_http_response()

        invoice_id = validated_response["invoiceID"]
        return self._serialize_payment_url(invoice_id)


class PaymentInterface:
    def __init__(self, http: Requests) -> None:
        self._payment_requests = PaymentRequests(http)

    def create_url_to_payment_page(self, robokassa_params: RobokassaParams) -> str:
        return self._payment_requests.create_url_to_payment_page(robokassa_params)


class PaymentUrlGenerator:
    def __init__(self, merchant_login: str, password: str, is_test: bool, hash_: Hash):
        self._merchant_login = merchant_login
        self._password = password
        self._is_test = is_test
        self._hash = hash_

        self._STATIC_URL = "https://auth.robokassa.ru/Merchant/Index.aspx"

    def _serialize_additional_params(self, default_prefix: str, params: dict) -> dict:
        return {f"{default_prefix}_{k}": v for k, v in params.items()}

    def _get_serialized_link_to_payment_page(self, url_params: Dict[str, Any]) -> str:
        return f"{self._STATIC_URL}?{urlencode(url_params)}"

    def _sort_urls(
        self, urls: Dict[str, str], success_url_method: str, fail_url_method: str
    ) -> Dict[str, str]:
        urls_plus_methods = {}
        for k, v in urls.items():
            if v is not None:
                urls_plus_methods[k] = f"{v}"
                if k == "SuccessUrl2":
                    method = ["SuccessUrl2Method", success_url_method]
                elif k == "FailUrl2":
                    method = ["FailUrl2Method", fail_url_method]
                else:
                    method = None
                if method is not None:
                    urls_plus_methods[method[0]] = method[1]
        return urls_plus_methods

    def generate_by_script(
        self,
        out_sum: float,
        default_prefix: str = "shp",
        result_url: Optional[str] = None,
        success_url: Optional[str] = None,
        success_url_method: Optional[str] = None,
        fail_url: Optional[str] = None,
        fail_url_method: Optional[str] = None,
        inv_id: Optional[int] = 0,
        description: Optional[str] = None,
        **kwargs,
    ):
        params = self._serialize_additional_params(default_prefix, kwargs)
        additional_params_for_url = sorted([f"{k}={v}" for k, v in params.items()])

        signature = Signature(
            merchant_login=self._merchant_login,
            out_sum=out_sum,
            inv_id=inv_id,
            password=self._password,
            result_url2=result_url,
            success_url2=success_url,
            success_url2_method=success_url_method,
            fail_url2=fail_url,
            fail_url2_method=fail_url_method,
            hash_=self._hash,
            additional_params=params,
        ).value

        urls_plus_methods = self._sort_urls(
            {
                "ResultUrl2": result_url,
                "SuccessUrl2": success_url,
                "FailUrl2": fail_url,
            },
            success_url_method,
            fail_url_method,
        )
        url_params = {
            k: v
            for k, v in [
                ("MerchantLogin", self._merchant_login),
                ("OutSum", out_sum),
                ("InvId", inv_id),
                ("Description", description),
                *urls_plus_methods.items(),
                ("SignatureValue", signature),
                ("IsTest", int(self._is_test)),
            ]
            if v is not None
        }
        return (
            f"{self._get_serialized_link_to_payment_page(url_params)}"
            f"&{"&".join(additional_params_for_url)}"
        )


class PaymentLink:
    def __init__(
        self,
        http: Requests,
        is_test: bool,
        hash_: Hash,
        merchant_login: str,
        password1: str,
    ) -> None:
        self._is_test = is_test
        self._merchant_login = merchant_login
        self._password = password1

        self._hash: Hash = hash_

        self.__http = http
        self._payment_interface = PaymentInterface(http)

        self._payment_generator = PaymentUrlGenerator(
            merchant_login=merchant_login,
            password=password1,
            is_test=is_test,
            hash_=self._hash,
        )

        self._STATIC_URL = "https://auth.robokassa.ru/Merchant/Index.aspx"

    def _create_signature(
        self, inv_id: Union[str, int], out_sum: Union[str, int, float]
    ) -> Signature:
        return Signature(
            merchant_login=self._merchant_login,
            password=self._password,
            inv_id=inv_id,
            out_sum=out_sum,
            hash_=self._hash,
        )

    def generate_by_script(
        self,
        out_sum: float,
        default_prefix: str = "shp",
        result_url: Optional[str] = None,
        success_url: Optional[str] = None,
        success_url_method: Optional[str] = None,
        fail_url: Optional[str] = None,
        fail_url_method: Optional[str] = None,
        inv_id: Optional[int] = 0,
        description: Optional[str] = None,
        **kwargs,
    ) -> str:
        return self._payment_generator.generate_by_script(
            out_sum=out_sum,
            default_prefix=default_prefix,
            result_url=result_url,
            success_url=success_url,
            success_url_method=success_url_method,
            fail_url=fail_url,
            fail_url_method=fail_url_method,
            inv_id=inv_id,
            description=description,
            **kwargs,
        )

    def create_link_to_payment_page_by_invoice_id(
        self,
        inv_id: Optional[Union[int, str]],
        out_sum: Union[float, int, str],
        description: str,
    ) -> str:
        return self._payment_interface.create_url_to_payment_page(
            RobokassaParams(
                inv_id=inv_id,
                out_sum=out_sum,
                description=description,
                merchant_login=self._merchant_login,
                is_test=self._is_test,
                signature_value=self._create_signature(inv_id, out_sum).value,
            )
        )


class Payment:
    def __init__(
        self,
        http: Requests,
        is_test: bool,
        hash_: Hash,
        merchant_login: str,
        password1: str,
        password2: str,
    ) -> None:
        self._is_test = is_test
        self._hash = hash_
        self._merchant_login = merchant_login
        self._password1 = password1
        self._password2 = password2

        self.__http = http

    @property
    def link(self) -> PaymentLink:
        return PaymentLink(
            http=self.__http,
            is_test=self._is_test,
            hash_=self._hash,
            merchant_login=self._merchant_login,
            password1=self._password1,
        )

    @property
    def check(self) -> SignaturesChecker:
        return SignaturesChecker(
            hash_=self._hash, password1=self._password1, password2=self._password2
        )
